from ontobio.ontol_factory import OntologyFactory

GENETIC_DISEASE='DOID:630'
MONOGENIC_DISEASE='DOID:0050177'

class Mondo():
    """Class to hold/query the mondo ontology""" 
    def __init__(self):
        ofactory = OntologyFactory()
        self.ont = ofactory.create('mondo')
        #This seems to be required to make the ontology actually load:
        _ = self.ont.get_level(0)
    def get_doid(self,identifier):
        """We have an identifier, and we are going to use MONDO to try to convert it to a DOID"""
        upper_id = identifier.upper()
        obj_ids = self.get_mondo_id(upper_id)
        #Are any of the ids we get back a DOID?
        doids = []
        for obj_id in obj_ids:
            if obj_id.startswith('DOID:'):
                doids.append(obj_id)
        if len(doids) > 0:
            return doids
        #Didn't get anything, so get the xrefs and find DOIDS
        for obj_id in obj_ids:
            xref_ids = self.ont.xrefs(obj_id)
            for xref_id in xref_ids:
                if xref_id.startswith('DOID:'):
                    doids.append( xref_id )
        return doids
    def get_mondo_id(self,obj_id):
        """Given an id, find the main key(s) that mondo uses for the id"""
        if self.ont.has_node(obj_id):
            obj_ids = [obj_id]
        else:
            obj_ids = self.ont.xrefs(obj_id, bidirectional=True)
        return obj_ids
    def has_ancestor(self,obj, term):
        """Given an object and a term in MONDO, determine whether the term is an ancestor of the object.
        
        The object is a KNode representing a disease.
        Some complexity arises because the identifier for the node may not be the id of the concept in mondo.
        the XRefs in mondo are checked for the object if it is not intially found, but this may return more 
        than one entity if multiple mondo entries map to the same.  

        Returns: boolean representing whether any mondo objects derived from the subject have the term as an
                         ancestor.
                 The list of Mondo identifiers for the object, which have the term as an ancestor"""
        #TODO: The return signature is funky, fix it.
        obj_id = obj.identifier
        obj_ids = self.get_mondo_id(obj_id)
        print ("obj ids {}".format (obj_ids))
        return_objects=[]
        for obj_id in obj_ids:
            ancestors = self.ont.ancestors(obj_id)
            if GENETIC_DISEASE in ancestors:
                return_objects.append( obj_id )
        return len(return_objects) > 0, return_objects
    def is_genetic_disease(self,obj):
        """Checks mondo to find whether the subject has DOID:630 as an ancestor"""
        return self.has_ancestor(obj, GENETIC_DISEASE)
    def is_monogenic_disease(self,obj):
        """Checks mondo to find whether the subject has DOID:0050177 as an ancestor"""
        return self.has_ancestor(obj, MONOGENIC_DISEASE)



import logging
import inspect
import json
import requests
import traceback
import unittest
import datetime
import os
import yaml
from bravado.client import SwaggerClient
from bravado.requests_client import RequestsClient

class Config:
    def __init__(self, config):
        if not config.startswith (os.sep):
            config = os.path.join (os.path.dirname (__file__), config)
        with open(config, 'r') as f:
            self.conf = yaml.safe_load(f)
    def get_service (self, service):
        return self.conf['translator']['services'][service]
    
class LoggingUtil(object):
    """ Logging utility controlling format and setting initial logging level """
    @staticmethod
    def init_logging (name, level=logging.INFO, format='long'):
        FORMAT = {
            "short" : '%(asctime)-15s %(levelname)s: %(message)s',
            "long"  : '%(asctime)-15s %(filename)s %(funcName)s %(levelname)s: %(message)s'
        }[format]
        logging.basicConfig (format=FORMAT, level=logging.WARNING) #DEBUG)
#        return logging.getLogger (name)
        logger = logging.getLogger (name)
        logger.setLevel(level)
        return logger
    
class Munge(object):
    @staticmethod
    def gene (gene):
        return gene.split ("/")[-1:][0] if gene.startswith ("http://") else gene
    
class SwaggerEndpoint(object):

    def __init__(self, swagger_endpoint_url):
        http_client = RequestsClient ()
        # TODO: fix ssl certificate.
        self.client = SwaggerClient.from_url(
            swagger_endpoint_url,
            http_client=http_client,
            config={
            #    "validate_requests" : False,
            #    "validate_responses" : False
                'use_models': False
            })

    def inspect (self):
        for name, obj in inspect.getmembers (self.client):
            print ("-- name: {0} obj: {1}".format (name, obj))
            for n, o in inspect.getmembers (obj):
                if (n.endswith ('_get') or n.endswith ('_post')):
                    print ("-- INVOKE: method-> {0} obj: {1}".format (n, o))



import graphene
from graphene import resolve_only_args
from .types import (
    ExposureScore,
    ExposureValue,
    ExposureCondition,
    Drug,
    GenePath
)
from .data import (
    get_exposure_score,
    get_exposure_value,
    get_patients,
    get_exposure_conditions,
    get_drugs_by_condition,
    get_genes_pathways_by_disease
)

class Query (graphene.ObjectType):

    exposure_score = graphene.Field (type=ExposureScore,
                                    exposureType  = graphene.String (),
                                    startDate     = graphene.String (),
                                    endDate       = graphene.String (),
                                    exposurePoint = graphene.String (),
                                    resolver      = get_exposure_score)
    
    exposure_value = graphene.Field (type=ExposureValue,
                                    exposureType  = graphene.String (),
                                    startDate     = graphene.String (),
                                    endDate       = graphene.String (),
                                    exposurePoint = graphene.String (),
                                    resolver      = get_exposure_value)

    exposure_conditions = graphene.List (of_type=ExposureCondition,
                                        chemicals = graphene.List(graphene.String))

    drugs_by_condition = graphene.List(of_type=Drug,
                                       conditions = graphene.List(graphene.String),
                                       resolver   = get_drugs_by_condition)

    gene_paths_by_disease = graphene.List (of_type=GenePath,
                                           diseases = graphene.List(graphene.String),
                                           resolver = get_genes_pathways_by_disease)
    
    def resolve_exposure_conditions (self, args, context, info):
        return get_exposure_conditions (self, args, context, info)
    
    
Schema = graphene.Schema(query=Query)


'''

{
  exposureValue(exposureType: "pm25", 
    		startDate: "2010-01-06", 
    		endDate: "2010-01-06", 
    	        exposurePoint: "35.9131996,-79.0558445") {
    value
  }
}

{
  exposureScore(exposureType: "pm25", 
    		startDate: "2010-01-06", 
    		endDate: "2010-01-06", 
    		exposurePoint: "35.9131996,-79.0558445") {
    value
  }
}

{
  exposureConditions (chemicals: [ "D052638" ] ) {
    chemical
    gene
    pathway
    pathName
    pathID
    human
  } 
}

{
  drugsByCondition (conditions: [ "d001249" ] ) {
    genericName
  } 
}

{
  genePathsByDisease (diseases: [ "d001249" ] ) {
    uniprotGene
    keggPath
    pathName
  } 
}

'''



import json
import os
import sys
from collections import defaultdict
from pprint import pformat
from pprint import pprint
from greent.client import GraphQL
from collections import namedtuple
from flask_testing import LiveServerTestCase
from greent.util import LoggingUtil
import networkx as nx
import networkx.algorithms as nxa

logger = LoggingUtil.init_logging (__file__)

class Vocab(object):
    root_kind         = 'http://identifiers.org/doi'
    
    # MESH
    mesh              = 'http://identifiers.org/mesh'
    mesh_disease_name = 'http://identifiers.org/mesh/disease/name'
    mesh_drug_name    = 'http://identifiers.org/mesh/drug/name'
    mesh_disease_id   = 'http://identifiers.org/mesh/disease/id'
    
    # Disease
    doid_curie          = "doid"
    doid                = "http://identifiers.org/doid"
    pharos_disease_name = "http://pharos.nih.gov/identifier/disease/name"

    # DRUG
    c2b2r_drug_name   = "http://chem2bio2rdf.org/drugbank/resource/Generic_Name"
    
    # TARGET
    c2b2r_gene        = "http://chem2bio2rdf.org/uniprot/resource/gene"
    
    # PATHWAY
    c2b2r_pathway     = "http://chem2bio2rdf.org/kegg/resource/kegg_pathway"

    # CELL
    hetio_cell        = "http://identifier.org/hetio/cellcomponent"

    # ANATOMY
    hetio_anatomy     = "http://identifier.org/hetio/anatomy"
    
    # Semantic equivalence

    def __init__(self):
        self.equivalence = defaultdict(lambda: [])
        self.equivalence[self.doid_curie] = [ self.doid ]
        
        # https://github.com/prefixcommons/biocontext/blob/master/registry/uber_context.jsonld
        uber_context_path = os.path.join(os.path.dirname(__file__), 'jsonld', 'uber_context.jsonld')
        with open (uber_context_path, 'r') as stream:
            self.equivalence = json.loads (stream.read ())["@context"]
            self.equivalence["MESH"] = self.equivalence ["MESH.2013"]

class NoTranslation (Exception):
    def __init__(self, message=None):
        super(NoTranslation, self).__init__(message)
        
class Translation (object):
    def __init__(self, obj, type_a=None, type_b=None, description="", then=None):
        print ("type_a: {0} type_b: {1}".format (type_a, type_b))
        self.obj = obj
        self.type_a = type_a
        self.type_b = type_b
        self.desc = description
        self.then = []
        self.response = None
    def __repr__(self):
        return "obj: {0} type_a: {1} type_b: {2} desc: {3} then: {4} response: {5}".format (
            self.obj, self.type_a, self.type_b, self.desc, "", #self.then,
            pformat (self.response [: min(len(self.response), 2)] if self.response else ""))

class TranslationAST(object):
    def __init__(self):
        self.stack = []
        self.from_type = defaultdict (lambda:[])
        self.to_type = defaultdict (lambda:[])
    def __repr__(self):
        text = []
        for t in self.stack:
            for r in t.response:
                text.append (r)
        return ", ".join (text)
    
class Mox(object):
    def __init__(self, thing, obj_type):
        self.thing = thing
        self.obj_type = obj_type

class Translator(object):

    def __init__(self, core):
        self.core = core
        self.vocab = Vocab ()
        
        # Domain translation
        self.translator_router = defaultdict (lambda: defaultdict (lambda: NoTranslation ()))
        self.translator_router[Vocab.mesh_disease_name] = {
            Vocab.mesh_drug_name : lambda disease: self.core.chemotext.disease_name_to_drug_name (disease)
        }
        self.translator_router[Vocab.doid][Vocab.mesh_disease_id]             = lambda doid:    self.core.disease_ontology.doid_to_mesh (doid.upper())
        self.translator_router[Vocab.c2b2r_drug_name][Vocab.c2b2r_gene]       = lambda drug:    self.core.chembio.drug_name_to_gene_symbol (drug)
        self.translator_router[Vocab.c2b2r_gene][Vocab.c2b2r_pathway]         = lambda gene:    self.core.chembio.gene_symbol_to_pathway (gene)
        self.translator_router[Vocab.c2b2r_gene][Vocab.pharos_disease_name]   = lambda gene:    self.core.pharos.target_to_disease (gene)
        self.translator_router[Vocab.doid][Vocab.c2b2r_gene]                  = lambda gene:    self.core.pharos.disease_to_target (gene)
        self.translator_router[Vocab.mesh][Vocab.root_kind]                   = lambda mesh_id: self.core.oxo.mesh_to_other (mesh_id)
        self.translator_router[Vocab.c2b2r_gene][Vocab.hetio_anatomy]         = lambda gene:    self.core.hetio.gene_to_anatomy (gene)
        self.translator_router[Vocab.c2b2r_gene][Vocab.hetio_cell]            = lambda gene:    self.core.hetio.gene_to_cell (gene)
        
    def resolve_id (self, an_id, domain):
        if not an_id in domain:
            candidate = an_id
            an_id = None
            if candidate in self.vocab.equivalence:
                for alternative in self.vocab.equivalence[candidate]:
                    if alternative in domain:
                        # Postpone the problem of synonymy
                        an_id = alternative
                        logger.debug ("Selected alternative id {0} for input {1}".format (an_id, candidate))
                        break
                    # Also, if all candidates turn out not to be in the domain, we could recurse to try synonyms for them
        return an_id

    def get_translator_op (self, type_a, type_b):
        logger.debug ("Translator request: {0} {1}".format (type_a, type_b))
        operator = None
        resolved_a = self.resolve_id (type_a, self.translator_router)
        resolved_b = self.resolve_id (type_b, self.translator_router[type_a])
        logger.debug ("resolved_a: {0} resolved_b: {1}".format (resolved_a, resolved_b))
        if resolved_a and resolved_b:
            operator = self.translator_router[resolved_a][resolved_b]
            print ("----------> operator: {0}".format (operator))
        return operator
    
    def translate (self, translation):
        result = None
        operator = self.get_translator_op (translation.type_a, translation.type_b)
        if operator != None and not isinstance (operator, NoTranslation):
            logger.info ("Translating: {0}".format (translation))
            translation.response = operator (translation.obj)
        return translation.response
    
    def translate0 (self, thing, type_a, type_b):
        logger.info ("Translator request thing: {0} {1} {2}".format (thing, type_a, type_b))
        result = None
        resolved_a = self.resolve_id (type_a, self.translator_router)
        resolved_b = self.resolve_id (type_b, self.translator_router[type_a])
        logger.info ("resolved_a: {0} resolved_b: {1}".format (resolved_a, resolved_b))
        if resolved_a and resolved_b:
            logger.info ("Translating {0} of type {1} to {2}".format (thing, type_a, type_b)) 
            result = self.translator_router[resolved_a][resolved_b] (thing)
            if isinstance (result, NoTranslation):
                raise NoTranslation ("No translation implemented from domain {0} to domain {1}".format (type_a, type_b))
        return result

    # Think:

    mox_resolve = {
        "drug"    : lambda x : Mox( x, Vocab.c2b2r_drug_name ),
        "gene"    : lambda x : Mox( x, Vocab.c2b2r_gene ),
        "pathway" : lambda x : Mox( x, Vocab.c2b2r_pathway ),
        "cell"    : lambda x : Mox( x, Vocab.hetio_cell ),
        "anatomy" : lambda x : Mox( x, Vocab.hetio_anatomy ),
        "disease" : lambda x : Mox( x, Vocab.doid ), #, Vocab.pharos_disease_name ),
        None      : lambda x : Mox( None, None )
    }
    
    def translate_chain (self, request, limit_breadth=2):
        '''
        Execute a chain of translations.

        :param request: A translation request of a term.
        :param limit_breadth: A limit on how many instances of results from one request to process in the subsequent request.
        :type request: dict
        :type limit_breadth: int
        :return: return Returns a list of translation objects with results.
        :rtype: Translation

        :Example:
        
        response = g.translator.translate_chain (request={
            "iri"     : "mox://drug/gene/pathway/cell/anatomy/disease",
            "drug"    : "Aspirin",
            "disease" : "Asthma"
        })
        '''
        elements = request['iri'].split ("mox://")[1].split ("/")
        ast = self.parse (elements, request)
        self.execute (ast, limit_breadth)
        return ast
    
    def execute (self, ast, limit_breadth):
        result = []
        for index, translation in enumerate (ast.stack):
            if translation.obj != None:
                output = self.translate (translation)
            else:
                antecedents = ast.to_type [translation.type_a]
                for pindex, previous in enumerate (antecedents):
                    if pindex > limit_breadth:
                        break
                    if previous.response == None:
                        continue
                    for item in previous.response:
                        # print ("-------> {0} {1}".format (type(item), item))
                        fragment = Translation (obj=item,
                                                type_a=translation.type_a,
                                                type_b=translation.type_b)
                        self.translate (fragment)
                        print ("%%%%>> {}".format (fragment))
                
    def execute_stack0 (self, stack, limit_breadth):
        for index, t in enumerate (stack):
            if t.obj == None and index > 0:
                last = stack [index - 1]
                logger.debug ("Last: {}".format (last))
                t.response = []
                if last.response:
                    for index, each in enumerate (last.response):
                        if limit_breadth < index:
                            break
                        t.response.append (self.translate (thing=each, type_a=last.type_b, type_b=t.type_b))
                        logger.info ("Translator response: {0}".format (pformat (t.response)))
                else:
                    logger.info ("last response is None for {}".format (last))
            else:
                t.response = self.translate (thing=t.obj, type_a=t.type_a, type_b=t.type_b)
                logger.info ("Translator response: {0}".format (pformat (t.response)))

    def get_mox (self, element, request):
        return self.mox_resolve [element](request[element] if element in request else None)
    
    def parse (self, elements, request):
        ast = TranslationAST ()
        for index1, e in enumerate (elements):
            mox = self.get_mox (e, request)
            root = None
            for index2, eprime in enumerate (elements):
                moxprime = self.get_mox (eprime, request)
                translation = Translation (obj=mox.thing, type_a=mox.obj_type, type_b=moxprime.obj_type)
                ast.from_type[mox.obj_type].append (translation)
                ast.to_type[moxprime.obj_type].append (translation)
                if root == None:
                    root = translation
                else:
                    root.then.append (translation)
                logger.debug(" --translation> {0}".format (translation))
                ast.stack.append (translation)
        return ast

    def parse_to_ast0 (self, elements, request):
        stack = []
        for index, e in enumerate (elements):
            mox = self.mox_resolve[e](request[e] if e in request else None)
            next_e = elements[index+1] if index < len(elements) - 1 else None
            print ("ne: {}".format (next_e))
            if next_e != None:
                value = request[next_e] if next_e in request else None
                next_mox = self.mox_resolve [next_e](value)
                logger.info (" next mox: {}".format (next_mox))
            else:
                next_mox = self.mox_resolve[None](None)
            translation = Translation (obj=mox.thing, type_a=mox.obj_type, type_b=next_mox.obj_type)
            logger.debug("Built translation: {0}".format (translation))
            stack.append (translation)
        return stack



from collections import defaultdict
from csv import DictReader
import logging
import pronto
import requests
import os

class DiseaseOntology (object):
    def __init__(self, obo_resource="http://purl.obolibrary.org/obo/doid.obo"):
        self.disease_ontology_data = 'doid.obo'
        self.initialized = False
        self.obo_resource = obo_resource
        self.pharos_map = None
    def load (self):
        if not os.path.exists (self.disease_ontology_data):
            url = self.obo_resource
            print ("Downloading disease ontology: {0}".format (url))
            r = requests.get(url, stream=True)
            with open(self.disease_ontology_data, 'wb') as f:
                for chunk in r.iter_content(chunk_size=1024): 
                    if chunk: # filter out keep-alive new chunks
                        f.write(chunk)                
        #print ("Loading disease ontology")
        self.ont = pronto.Ontology (self.disease_ontology_data)
        ''' print (ont.json) '''


        self.doid_to_mesh_map = defaultdict(lambda:[])
        for term in self.ont:
            xref = None
            if 'xref' in term.other:
                for xref in term.other['xref']:
                    if xref.startswith ("MESH:"):
                        #print ("doid: {0} -> {1}".format (term.id.upper (), xref))
                        self.doid_to_mesh_map[term.id.upper ()].append (xref)
        self.initialized = True

    def doid_to_mesh (self, doid):
        if not self.initialized:
            self.load ()
        return self.doid_to_mesh_map [doid]
    def doid_to_pharos(self,doid):
        pmap = defaultdict(list)
        with open(os.path.join(os.path.dirname(__file__), 'pharos.id.txt'),'r') as inf:     #'pharos.id.txt','r') as inf:
            rows = DictReader(inf,dialect='excel-tab')
            for row in rows:
                if row['DOID'] != '':
                    doidlist = row['DOID'].split(',')
                    for d in doidlist:
                        pmap[d].append(row['PharosID'])
        pharos_list = pmap[doid.identifier]
        if len(pharos_list) == 0:
            logging.getLogger('application').warn('Unable to translate %s into Pharos ID' % doid)
            return None
        return pharos_list
    
    def doid_to_pharos0(self, doid):
        """Convert a subject with a DOID into a Pharos Disease ID"""
        #TODO: This relies on a pretty ridiculous caching of a map between pharos ids and doids.  
        #      As Pharos improves, this will not be required, but for the moment I don't know a better way.
        if self.pharos_map == None:
            self.pharos_map = {}
            with open(os.path.join(os.path.dirname(__file__), 'pharos.id.txt'),'r') as inf:
                rows = DictReader(inf,dialect='excel-tab')
                for row in rows:
                    if row['DOID'] != '':
                        self.pharos_map[row['DOID']] = row['PharosID']
        return self.pharos_map[doid] if doid in self.pharos_map else None
    
if __name__ == "__main__":
    do = DiseaseOntology ()
    print (do.doid_to_mesh ("DOID:2841"))
    print (do.doid_to_pharos("DOID:2841"))



import json
import traceback
import unittest
import datetime
from string import Template
from pprint import pprint
from greent.util import SwaggerEndpoint
from greent.util import LoggingUtil
from greent.util import Config

class Endotype(SwaggerEndpoint):
    def __init__(self, url):
        super(Endotype, self).__init__(url)
    @staticmethod
    def create_request (dob, model_type, race, sex, visits):
        return {
            "date_of_birth": dob,
            "model_type": model_type,
            "race": race,
            "sex": sex,
            "visits": visits
        }
    @staticmethod
    def create_exposure (exposure_type, units, value):
        return {
            "exposure_type": exposure_type,
            "units": units,
            "value": value
        }
    @staticmethod
    def create_visit (icd_codes, lat, lon, time, visit_type, exposures):
        return {
            "exposures"  : exposures,
            "icd_codes"  : icd_codes,
            "lat"        : lat,
            "lon"        : lon,
            "time"       : time,
            "visit_type" : visit_type
        }
    def get_endotype (self, request):
        print (json.dumps (request))
        r = self.client.endotypes.endotypes_post (input=request).result()
        print (r)
        return r['output'] if 'output' in r else None
    
if __name__ == "__main__":
    conf = Config ('config.yaml')
    e = Endotype (conf.get_service("endotype")["url"])
    exposures = list(map(lambda exp : e.create_exposure (**exp), [{
        "exposure_type": "pm25",
        "units"        : "",
        "value"        : 2
    }]))
    visits = list(map(lambda v : e.create_visit(**v), [{
            "icd_codes"  : "ICD9:V12,ICD9:E002",
            "lat"        : "20",
            "lon"        : "20",
            "time"       : "2017-10-12 21:12:29",
            "visit_type" : "INPATIENT",
            "exposures"  : exposures
        }]))
    request = e.create_request (dob= "2017-10-04", model_type="M0", race="1", sex="M", visits = visits)
    print (json.dumps (e.get_endotype (request), indent=2))
           



import logging
import unittest
import json
import logging
import os
import traceback
import networkx as nx
import networkx.algorithms as nxa
import operator
from networkx.exception import NetworkXNoPath
from greent.util import LoggingUtil
from pprint import pformat

logger = LoggingUtil.init_logging (__file__, logging.DEBUG)

class Translation (object):
    def __init__(self, obj, type_a=None, type_b=None, description="", then=None):
#        print ("Translation(obj:{0}, type_a: {1} type_b: {2})".format (obj, type_a, type_b))
        self.obj = obj
        self.type_a = type_a
        self.type_b = type_b
        self.desc = description
        self.then = []
        self.response = None
    def __repr__(self):
        return "Translation(obj: {0} type_a: {1} type_b: {2} desc: {3} then: {4} response: {5})".format (
            self.obj, self.type_a, self.type_b, self.desc, "", #self.then,
            pformat (self.response [: min(len(self.response), 2)] if self.response else ""))

default_router_config = {
    "@curie" : {
        "DOID" : "doid",
        "MESH" : "mesh"
    },
    "@vocab" : {
        "c2b2r_drug_name"     : "http://chem2bio2rdf.org/drugbank/resource/Generic_Name",
        "c2b2r_drug_id"       : "http://chem2bio2rdf.org/drugbank/resource/drugbank_drug",
        "c2b2r_gene"          : "http://chem2bio2rdf.org/uniprot/resource/gene",
        "c2b2r_pathway"       : "http://chem2bio2rdf.org/kegg/resource/kegg_pathway",
        "doid"                : "http://identifiers.org/doid",
        "genetic_condition"   : "http://identifiers.org/mondo/gentic_condition",
        "hetio_anatomy"       : "http://identifier.org/hetio/anatomy",
        "hetio_cell"          : "http://identifier.org/hetio/cellcomponent",
        "hgnc_id"             : "http://identifier.org/hgnc/gene/id",
        "mesh"                : "http://identifiers.org/mesh",
        "mesh_disease_id"     : "http://identifiers.org/mesh/disease/id",
        "mesh_disease_name"   : "http://identifiers.org/mesh/disease/name",
        "mesh_drug_name"      : "http://identifiers.org/mesh/drug/name",
        "pharos_disease_id"   : "http://pharos.nih.gov/identifier/disease/id",
        "pharos_disease_name" : "http://pharos.nih.gov/identifier/disease/name",
        "root_kind"           : "http://identifiers.org/doi"
    },
    "@transitions" : {
        "mesh_disease_name" : {
            "mesh_drug_name"      : { "op" : "chemotext.disease_name_to_drug_name" }
        },
        "mesh_disease_id"   : {
            "c2b2r_drug_id"       : { "op" : "chembio.get_drugs_by_condition_graph" }
        },
        "doid"              : {
            "mesh_disease_id"     : { "op" : "disease_ontology.doid_to_mesh"   },
            "pharos_disease_id"   : { "op" : "disease_ontology.doid_to_pharos" },
            "hgnc_id"             : { "op" : "pharos.disease_get_gene"         }
        },
        "c2b2r_drug_name"   : {
            "c2b2r_gene"          : { "op" : "chembio.drug_name_to_gene_symbol" }            
        },
        "c2b2r_gene"        : {
            "c2b2r_pathway"       : { "op" : "chembio.gene_symbol_to_pathway" },
            "pharos_disease_name" : { "op" : "pharos.target_to_disease" },
            "hetio_anatomy"       : { "op" : "hetio.gene_to_anatomy" },
            "hetio_cell"          : { "op" : "hetio.gene_to_cell" }
        },
        "hgnc_id"           : {
            "genetic_condition"   : { "op" : "biolink.gene_get_genetic_condition" }
        },
        "pharos_disease_id" : {
            "hgnc_id"             : { "op" : "pharos.disease_get_gene" }
        },
        "mesh"              : {
            "root_kind"           : { "op" : "oxo.mesh_to_other" }
        }
    }
}

class Rosetta:
    def __init__(self, greentConf, config=default_router_config):
        from greent.core import GreenT
        self.core = GreenT (config=greentConf)
        self.g = nx.DiGraph ()
        self.vocab = config["@vocab"]
        for k in self.vocab:
            self.g.add_node (self.vocab[k])
        self.curie = config["@curie"]
        # import cmungall's uber context.
        with open(os.path.join (os.path.dirname (__file__), "jsonld", "uber_context.jsonld"), "r") as stream:
            uber = json.loads (stream.read ())
            context = uber['@context']
            for k in context:
                self.curie[k] = context[k]
                self.vocab[k] = context[k]
        transitions = config["@transitions"]
        for L in transitions:
            for R in transitions[L]:
                logger.debug ("  +edge: {0} {1} {2}".format (L, R, transitions[L][R]))
                self.g.add_edge (L, R, data=transitions[L][R])
                self.g.add_edge (self.vocab[L], self.vocab[R], data=transitions[L][R])
    def guess_type (self, thing, source):
        if thing and not source and ':' in thing:
            curie = thing.upper ().split (':')[0]
            if curie in self.curie:
                source = self.curie[curie]
        if not source.startswith ("http://"):
            source = self.vocab[source] if source in self.vocab else None
        return source
    def get_transitions (self, source, dest):
        #logger.debug ("get-transitions: {0} {1}".format (source, dest))
        transitions = []
        try:
            paths = nxa.all_shortest_paths (self.g, source=source, target=dest)
            for path in paths:
                logger.debug ("  path: {0}".format (path))
                steps = list(zip(path, path[1:]))
                logger.debug ("  steps: {}".format (steps))
                for step in steps:
                    logger.debug ("    step: {}".format (step))
                    edges = self.g.edges (step, data=True)
                    for e in edges:
                        if step[1] == e[1]:
                            logger.debug ("      trans: {0}".format (e))
                            transition = e[2]['data']['op']
                            transitions.append (transition)
        except NetworkXNoPath:
            pass
        except KeyError:
            pass
        return transitions
    def translate (self, thing, source, target):
        if not thing:
            return None
        source = self.guess_type (thing, source)
        target = self.guess_type (None, target)
        transitions = self.get_transitions (source, target)
        last = thing
        #if len(transitions) == 0:
        #    logger.debug ("   [transitions:{2}] {0}->{1}".format (source, target, len(transitions)))
        #else:
        if len(transitions) > 0:
            logger.debug ("              [transitions:{3}] {0}->{1} {2}".format (source, target, transitions, len(transitions)))
        for transition in transitions:
            try:
                data_op = operator.attrgetter(transition)(self.core) 
                this = data_op (last)
                result_text = this[:min(len(this),3)] if this else None if isinstance(this, list) else this
                logger.debug ("              invoke: {0}({1}) => {2}".format (transition, last, result_text))
                last = this
                if last == None:
                    break
            except:
                traceback.print_exc ()
        return last if len(transitions) > 0 else None

if __name__ == "__main__":
    translator = Rosetta ()
    test = {
        "c2b2r_gene" : "pharos_disease_name",
        "c2b2r_gene" : "hetio_cell",
        "mesh"       : "root_kind",
        "doid"       : "hgnc_id"
    }
#    for t in test:
#        print ("transitions: {}".format (translator.get_transitions (t, test[t])))
        
    things = [
        "DOID:0060728",
        "DOID:0050777",
        "DOID:2841"
    ]
    quiet = [ "connectionpool", "requests" ]
    for q in quiet:
        logging.getLogger(q).setLevel(logging.WARNING)
    for t in things:
#        hgnc = translator.translate (t, None, translator.vocab["hgnc_id"])
#        m    = translator.translate (t, None, translator.vocab["mesh_disease_id"])
        #print (m)

        logging.getLogger("chembio").setLevel (logging.DEBUG)
        m = 'MESH:D001249'
        d    = translator.translate (m, translator.vocab["mesh_disease_id"], translator.vocab["c2b2r_drug_id"])

        print (d)
        '''
        g    = translator.translate (t, None, translator.vocab["c2b2r_gene"])
        print ("gene: {}".format (g))
        p    = translator.translate (g, "c2b2r_gene", translator.vocab["hgnc_id"])
        c    = translator.translate (g, "c2b2r_gene", translator.vocab["hetio_cell"])
        a    = translator.translate (g, "c2b2r_gene", translator.vocab["hetio_anatomy"])
        '''



import json
import unittest
from pprint import pprint
from greent.neo4j import Neo4JREST
from greent.util import Munge

class HetIO(Neo4JREST):

    def __init__(self, url="https://neo4j.het.io"):
        super (HetIO, self).__init__(url)

    def munge_gene (self, gene):
        return gene.split ("/")[-1:][0] if gene.startswith ("http://") else gene

    def gene_to_anatomy (self, gene):
        return self.query (
            "MATCH (a:Anatomy)-[ar]-(g:Gene) WHERE g.name='%s' RETURN a, ar, g LIMIT 5" % Munge.gene (gene),
            labels=['Anatomy'],
            node_properties=['identifier'])

    def gene_to_cell (self, gene):
        gene = self.munge_gene (gene)
        return self.query (
            "MATCH (g:Gene)-[r]-(c:CellularComponent) WHERE g.name='%s' RETURN g, r, c LIMIT 200" % Munge.gene (gene),
            labels=['CellularComponent'],
            node_properties=['identifier'])

class TestHetIO(unittest.TestCase):

    h = HetIO ()
    
    def test_anatomy (self):
        pprint (self.h.gene_to_anatomy ('TP53'))

    def test_cell (self):
        pprint (self.h.gene_to_cell ('TP53'))
    
if __name__ == '__main__':
    unittest.main ()




from string import Template
import json
import os
import logging
from greent.triplestore import TripleStore
from greent.util import LoggingUtil
from reasoner.graph_components import KEdge, KNode
from pprint import pprint

logger = LoggingUtil.init_logging (__file__, logging.DEBUG)

class ChemBioKS(object):
    """ Generic service endpoints for medical and bio-chemical data. This set
        comprises portions of chem2bio2rdf, Monarch, and CTD environmental
        exposures."""
    def __init__(self, triplestore):
        self.triplestore = triplestore

    #@provenance()
    def query_chembio (self, query):
        """ Execute and return the result of a SPARQL query. """
        return self.triplestore.execute_query (query)

    def get_exposure_conditions (self, chemicals):
        """ Identify conditions (MeSH IDs) triggered by the specified stressor
            agent ids (also MeSH IDs).

        :param chemicals: List of IDs for substances of interest.
        :type chemicals: list of MeSH IDs, eg. D052638
        """
        id_list = ' '.join (list(map (lambda d : "( mesh:{0} )".format (d),
                            chemicals)))
        text = self.triplestore.get_template ("ctd_gene_expo_disease").\
            safe_substitute (chemicals=id_list)
        results = self.triplestore.execute_query (text)
        return list(map (lambda b : {
            "chemical" : b['chemical'].value,
            "gene"     : b['gene'].value,
            "pathway"  : b['kegg_pathway'].value,
            "pathName" : b['pathway_name'].value,
            "pathID"   : b['pathway_id'].value,
            "human"    : '(human)' in b['pathway_name'].value
        },
                         results.bindings))

    def get_drugs_by_condition (self, conditions):
        """ Get drugs associated with a set of conditions.

        :param conditions: Conditions to find associated drugs for.
        :type conditions: List of MeSH IDs for conditions, eg.: D001249
        """
        if not isinstance (conditions,list):
            conditions = [ conditions ]

        #condition_list = ' '.join (list(map (lambda d : "( mesh:{0} )".format (d), conditions)))
        conditions = list(map(lambda v : v.replace ("MESH:", "mesh:"), conditions))
        prefix = "mesh:"
        if any(map(lambda v : v.startswith(prefix), conditions)):
            prefix = ""
        '''
        condition_list = ' '.join (list(map (lambda d : "( {0}{1} )".format (prefix, d) , conditions)))
        text = self.triplestore.get_template ("get_drugs_by_disease").substitute (conditions=condition_list)
        print (text)
        results = self.triplestore.execute_query (text)
        return list(map (lambda b : b['generic_name'].value, results.bindings))
        '''

        #print ("-------- {}".format (conditions))
        condition_list = ', '.join (list(map (lambda d : " {0}{1} ".format (prefix, d) , conditions)))
        result = self.triplestore.query_template (
            inputs = { "diseaseIds" : condition_list.lower () },
            outputs = [ 'drugID', 'drugGenericName', 'pubChemCID', 'diseasePMIDs' ],
            template_text="""
prefix mesh:           <http://bio2rdf.org/mesh:> 
prefix ctd:            <http://chem2bio2rdf.org/ctd/resource/>
prefix db_resource:    <http://chem2bio2rdf.org/drugbank/resource/>
select ?drugID ?drugGenericName ?diseasePMIDs ?ctdChemDis ?pubChemCID where {
    values ( ?diseaseId ) { ( $diseaseIds ) }
    ?ctdChemDis  ctd:cid                        ?pubChemCID;
                 ctd:diseaseid                  ?diseaseId;
                 ctd:pubmedids                  ?diseasePMIDs.
    ?dbInter     db_resource:Name               ?name ;
	         db_resource:DBID               ?drugID .
    ?drugID      db_resource:CID                ?pubChemCID ;
  	         db_resource:Generic_Name       ?drugGenericName .
}""")
        return result

    def get_drugs_by_condition_graph (self, conditions):
        drugs = self.get_drugs_by_condition (conditions)
        '''
        for e in edges:
            print (e.to_json ())
        for n in nodes:
            print (n.to_json ())
        '''
        '''
        return KGraph (nodes = [ KNode (r['drugID'].split('/')[-1:][0], "http://chem2bio2rdf.org/drugbank/resource/drugbank_drug", r['drugGenericName']) for r in drugs ],
                       edges = [ KEdge ('chem2bio2rdf', 'conditionToDrug', { 'cid' : r['pubChemCID'], 'pmids' : r['diseasePMIDs'] }) for r in drugs ])
        '''
    
        results = []
        for r in drugs:
            edge = KEdge ('c2b2r', 'conditionToDrug',
                          { 'cid' : r['pubChemCID'], 'pmids' : r['diseasePMIDs'] })
            node = KNode (r['drugID'].split('/')[-1:][0],
                          "http://chem2bio2rdf.org/drugbank/resource/drugbank_drug",
                          r['drugGenericName'])
            results.append ( (edge, node) )
        print ("chembio drugs by condition: {}".format (results))
        return results
                
    def get_genes_pathways_by_disease (self, diseases):
        """ Get genes and pathways associated with specified conditions.

        :param diseases: List of conditions designated by MeSH ID.
        :return: Returns a list of dicts containing gene and path information.
        """
        diseaseMeshIDList = ' '.join (list(map (lambda d : "( mesh:{0} )".format (d), diseases)))
        text = self.triplestore.get_template ("genes_pathways_by_disease").safe_substitute (diseaseMeshIDList=diseaseMeshIDList)
        results = self.triplestore.execute_query (text)
        return list(map (lambda b : {
            "uniprotGene" : b['uniprotGeneID'].value,
            "keggPath"    : b['keggPath'].value,
            "pathName"    : b['pathwayName'].value,
            "human"       : '(human)' in b['pathwayName'].value
        },
        results.bindings))

    
    def get_drug_gene_disease (self, disease_name, drug_name):
        """ Identify targets and diseases assocaited with a drug name.
        :param disease_name: MeSH name of a disease condition.
        :type str: String
        :param drug_name: Name of a drug.
        :type str: String
        """
        text = self.triplestore.get_template ("drug_gene_disease").safe_substitute (
            diseaseName=disease_name,
            drugName=drug_name)
        results = self.triplestore.execute_query (text)
        return list(map (lambda b : {
            "uniprotSymbol" : b['uniprotSym'].value,
            "diseaseId"     : b['diseaseID'].value
        }, results.bindings))


    def drug_name_to_gene_symbol (self, drug_name):
#        result = self.query (
        result = self.triplestore.query_template (
            inputs = { "drugName" : drug_name },
            outputs = [ 'uniprotSym' ],
            template_text="""
prefix ctd:            <http://chem2bio2rdf.org/ctd/resource/>
prefix db_resource:    <http://chem2bio2rdf.org/drugbank/resource/>
select ?drugGenericName ?uniprotSym where {
    values ( ?drugName ) { ( "$drugName" ) }
    ?ctdChemGene ctd:cid                        ?pubChemCID;
                 ctd:gene                       ?uniprotSym .
    ?drugID      db_resource:CID                ?pubChemCID ;
  	         db_resource:Generic_Name       ?drugGenericName .
  filter regex(lcase(str(?drugGenericName)), lcase(?drugName))
}""")
        #logger.debug (result)
        return list(map(lambda r : r['uniprotSym'], result)) #result
    
    def gene_symbol_to_pathway (self, uniprot_symbol):
        return self.triplestore.query_template (
            inputs = { "uniprotSymbol" : uniprot_symbol },
            outputs = [ "keggPath" ],
            template_text="""
prefix kegg:           <http://chem2bio2rdf.org/kegg/resource/>
prefix pharmgkb:       <http://chem2bio2rdf.org/pharmgkb/resource/>
prefix ctd:            <http://chem2bio2rdf.org/ctd/resource/>
select ?ctdGene ?uniprotID ?pathwayName ?keggPath where {
    values ( ?ctdGene ) { ( <$uniprotSymbol> ) }
    ?keggPath    kegg:protein    ?uniprotID ; kegg:Pathway_name ?pathwayName .
    ?pharmGene   pharmgkb:Symbol ?ctdGene ; pharmgkb:UniProt_Id ?uniprotID.
    ?ctdChemGene ctd:gene        ?ctdGene.
} LIMIT 500
""")

    '''
    def query (self, query_template, output_fields, input_fields=[]):
        query_template = Template(query_template)
        query_text = query_template.safe_substitute (**input_fields)
        logger.debug (query_text)
        query_results = self.triplestore.execute_query (query_text)
        logger.debug ("query bindings: {0}".format (query_results.bindings))
        result = set ()
        for b in query_results.bindings:
            logger.debug (b)
            for f in output_fields:
                result.add (b[f].value)
        return result
#        return { key : value.value for (key, value) in query_results.bindings }
    '''



from collections import defaultdict
import pronto
import requests
import os

class HumanPhenotypeOntology (object):
    def __init__(self):
        hpo_data = 'hpo.obo'
        if not os.path.exists (hpo_data):
            url = "http://purl.obolibrary.org/obo/hp.obo"
            print ("Downloading human phenotype ontology: {0}".format (url))
            r = requests.get(url, stream=True)
            with open(hpo_data, 'wb') as f:
                for chunk in r.iter_content(chunk_size=1024): 
                    if chunk: # filter out keep-alive new chunks
                        f.write(chunk)
        print ("Loading human phenotype ontology")
        self.ont = pronto.Ontology (hpo_data)
        ''' print (ont.json) '''

    def get_term (self, term_id):
        return self.ont[term_id]
    

hpo = HumanPhenotypeOntology ()

term = hpo.get_term ('HP:0002099')

def dump_term (t):
    print (t)
    #print (t['def'])
    for c in t.rchildren ():
        print ("  child: {}".format (c))
        #print ("  child: {}".format (c['def']))
        dump_term (c)
    for r in t.relations:
        print ("relation: {}".format (r))
        
        
dump_term (term)

#print (do.doid_to_mesh ["DOID:2841"])



import asyncio
import concurrent.futures
import requests
import json
import logging
import sys
import traceback
from collections import defaultdict
from csv import DictReader
from greent.disease_ont import DiseaseOntology
from greent.util import Munge
from greent.util import LoggingUtil
from greent.async import AsyncUtil
from reasoner.graph_components import KEdge, KNode

logger = LoggingUtil.init_logging (__name__, logging.DEBUG)

class Pharos(object):
    def __init__(self, url="https://pharos.nih.gov/idg/api/v1"):
        self.url = url
        self.disease_ontology = DiseaseOntology ()
    def request (self, url):
        #print ("pharos url: {}".format (url))
        response = None
        try:
            response = requests.get (url).json ()
        except:
            traceback.print_exc ()
        return response
        
    def query (self, query):
        return self.request ("{0}/{1}".format (self.url, query))
    def target_to_id (self, target):
        target_map = self.query (query="targets/{0}".format (Munge.gene (target)))
        return target_map["id"] if "id" in target_map else None
    def target_to_disease (self, target_sym):
        target_id = self.target_to_id (Munge.gene (target_sym))
        links = self.query (query="targets({0})/links".format (target_id))
        result = []
        for k in links:
            if k['kind'] == "ix.idg.models.Disease":
                for p in k['properties']:
                    #print (p)
                    if p['label'] == 'IDG Disease':
                        result.append (p['term'])
        return result
    
    def disease_map (self, disease_id):
        return self.query (query="diseases({0})".format (disesase_id))

    def make_doid_id (self, obj):
        if not obj:
            return None
        result = obj
        if isinstance (obj, KNode):
            result = obj.identifier
        if isinstance (result, list):
            if len(result) == 1:
                result = result[0]
        if result:
            if isinstance(result, str):
                if result.startswith ('DOID:'):
                    result = result.replace ('DOID:', '')
        return result

    def translate(self, subject_node):
        """Convert a subject with a DOID into a Pharos Disease ID"""
        #TODO: This relies on a pretty ridiculous caching of a map between pharos ids and doids.  
        #      As Pharos improves, this will not be required, but for the moment I don't know a better way.
        pmap = defaultdict(list)
        with open('pharos.id.txt','r') as inf:
            rows = DictReader(inf,dialect='excel-tab')
            for row in rows:
                if row['DOID'] != '':
                    doidlist = row['DOID'].split(',')
                    for d in doidlist:
                        pmap[d].append(row['PharosID'])
        doid = subject_node.identifier
        pharos_list = pmap[doid]
        if len(pharos_list) == 0:
            logging.getLogger('application').warn('Unable to translate %s into Pharos ID' % doid)
            return None
        return pharos_list

    def target_to_hgnc(self, target_id):
        """Convert a pharos target id into an HGNC ID.
        
        The call does not return the actual name for the gene, so we do not provide it.
        There are numerous other synonyms that we could also cache, but I don't see much benefit here"""
        result = None
        #node = self.make_doid_id (target_id)
        try:
            #print ('https://pharos.nih.gov/idg/api/v1/targets(%s)/synonyms' % target_id)
            r = requests.get('https://pharos.nih.gov/idg/api/v1/targets(%s)/synonyms' % target_id)
            result = r.json()
            for synonym in result:
                if synonym['label'] == 'HGNC':
                    result = synonym['term']
        except:
            #traceback.print_exc ()
            pass
        return result

    #TODO: assuming a DOID, not really valid
    #TODO: clean up, getting ugly
    def disease_get_gene(self, subject):
        pharosids = self.translate (subject)
        print ("pharos ids: {}".format (pharosids))
        original_edge_nodes=[]
        '''
        async def parallel_requests (urls, process_response, degree=20):
            with concurrent.futures.ThreadPoolExecutor(max_workers=degree) as executor:
                loop = asyncio.get_event_loop()
                futures = [
                    loop.run_in_executor(
                        executor, 
                        requests.get, 
                        url
                    )
                    for url in urls
                ]
                for response in await asyncio.gather(*futures):
                    process_response (response)
        '''
        def process_pharos_response (r):
            result = r.json()
            for link in result['links']:
                #logger.debug ("link %s", link)
                if link['kind'] != 'ix.idg.models.Target':
                    logger.info('Pharos disease returning new kind: %s' % link['kind'])
                else:
                    pharos_target_id = int(link['refid'])
                    pharos_edge = KEdge( 'pharos', 'queried', {'properties': link['properties']} )
                    original_edge_nodes.append( (pharos_edge, pharos_target_id) )

        AsyncUtil.execute_parallel_requests (
            urls=[ "https://pharos.nih.gov/idg/api/v1/diseases(%s)?view=full" % p for p in pharosids ],
            response_processor=process_pharos_response,
            chunk_size=30)
        
        '''
        urls = []
        chunks = 20
        for pharosid in pharosids:
            logger.debug ('pharos> https://pharos.nih.gov/idg/api/v1/diseases(%s)?view=full' % pharosid)
            urls.append ('https://pharos.nih.gov/idg/api/v1/diseases(%s)?view=full' % pharosid)
            if len(urls) % chunks == 0:
                loop = asyncio.get_event_loop()
                loop.run_until_complete(parallel_requests(urls, process_response=process_pharos_response))                
        if len(urls) > 0:
            loop = asyncio.get_event_loop()
            loop.run_until_complete(parallel_requests(urls, process_response=process_pharos_response))
        '''
        #Pharos returns target ids in its own numbering system. Collect other names for it.
        resolved_edge_nodes = []
        index = 0
        for edge, pharos_target_id  in original_edge_nodes:
            #logger.debug ("edge: %s", edge)
            hgnc = self.target_to_hgnc(pharos_target_id)
            index = index + 1
            if hgnc is not None:
                #logger.debug ("making hgnc node {0}".format (hgnc))
                hgnc_node = KNode(hgnc, 'G')
                if index < 10:
                    logger.debug ("            hgnc-node: %s", hgnc_node)
                resolved_edge_nodes.append((edge,hgnc_node))
            else:
                logging.getLogger('application').warn('Did not get HGNC for pharosID %d' % pharos_target_id)

        return resolved_edge_nodes


    def disease_get_gene0(self, subject):
        pharosids = self.translate (subject)
        print ("pharos ids: {}".format (pharosids))
        original_edge_nodes=[]
        for pharosid in pharosids:
            logger.debug ('pharos> https://pharos.nih.gov/idg/api/v1/diseases(%s)?view=full' % pharosid)
            r = requests.get('https://pharos.nih.gov/idg/api/v1/diseases(%s)?view=full' % pharosid)
            result = r.json()
            for link in result['links']:
                if link['kind'] != 'ix.idg.models.Target':
                    logger.info('Pharos disease returning new kind: %s' % link['kind'])
                else:
                    pharos_target_id = int(link['refid'])
                    pharos_edge = KEdge( 'pharos', 'queried', {'properties': link['properties']} )
                    original_edge_nodes.append( (pharos_edge, pharos_target_id) )

        #Pharos returns target ids in its own numbering system. Collect other names for it.
        resolved_edge_nodes = []
        index = 0
        for edge, pharos_target_id  in original_edge_nodes:
            #logger.debug ("edge: %s", edge)
            hgnc = self.target_to_hgnc(pharos_target_id)
            index = index + 1
            if hgnc is not None:
                #logger.debug ("making hgnc node {0}".format (hgnc))
                hgnc_node = KNode(hgnc, 'G')
                if index < 10:
                    logger.debug ("            hgnc-node: %s", hgnc_node)
                resolved_edge_nodes.append((edge,hgnc_node))
            else:
                logging.getLogger('application').warn('Did not get HGNC for pharosID %d' % pharos_target_id)

        return resolved_edge_nodes


#Poking around on the website there are about 10800 ( a few less )
def build_disease_translation():
    """Write to disk a table mapping Pharos disease ID to DOID (and other?) so we can reverse lookup"""
    with open('pharos.id.txt','w') as pfile:
        pfile.write('PharosID\tDOID\n')
        for pharosid in range(1,10800):
            r = requests.get('https://pharos.nih.gov/idg/api/v1/diseases(%d)/synonyms' % pharosid).json()
            doids = []
            for synonym in r:
                if synonym['label'] == 'DOID':
                    doids.append(synonym['term'])
            if len(doids) > 1:
                #print(doids)
                import json
                #print( json.dumps(r,indent=4) )
                exit()
            elif len(doids) == 0:
                doids.append('')
            pfile.write('%d\t%s\n' % (pharosid, doids[0]))

def test_disese_gene_for_output():
    """Call a function so that we can examine the output"""
    pharosid=455
    r = requests.get('https://pharos.nih.gov/idg/api/v1/diseases(%d)?view=full' % pharosid)
    result = r.json()
    import json
    with open('testpharos.txt','w') as outf:
        json.dump(result,outf,indent=4)

def test_hgnc_for_output():
    """Call a function so that we can examine the output"""
    pharosid=91
    r = requests.get('https://pharos.nih.gov/idg/api/v1/targets(%d)/synonyms' % pharosid)
    result = r.json()
    import json
    with open('testpharos.txt','w') as outf:
        json.dump(result,outf,indent=4)
    
def test ():
#    with open("pharos.txt", "r") as stream:
#        obj = json.loads (stream.read ())
#        print (json.dumps (obj, indent=2))        
    pharos = Pharos ()
    diseases = pharos.target_to_disease ("CACNA1A")
    print (diseases)


def test_async ():
    async def amain():
        with concurrent.futures.ThreadPoolExecutor(max_workers=20) as executor:            
            loop = asyncio.get_event_loop()
            futures = [
                loop.run_in_executor(
                    executor, 
                    requests.get, 
                    'http://example.org/'
                )
                for i in range(20)
            ]
            for response in await asyncio.gather(*futures):
                print ("a {}".format (response.text))            
                pass
    loop = asyncio.get_event_loop()
    loop.run_until_complete(amain())
    #amain ()
    sys.exit (0)    
        
if __name__ == '__main__':
    test_async ()


    test ()
    build_disease_translation ()
    



import inspect
import json
import requests
import traceback
import unittest
import datetime
from bravado.client import SwaggerClient
from bravado.requests_client import RequestsClient
from string import Template
from pprint import pprint
from greent.util import SwaggerEndpoint

class CMAQ (SwaggerEndpoint):

    def __init__(self, swagger_endpoint_url=""):
        super (CMAQ, self).__init__(swagger_endpoint_url)
        self.url = swagger_endpoint_url
    
    def get_meta (self):
        return self.client.cmaq.get_cmaq ().result (timeout = 10)
    
    def get_scores (self, start_date, end_date, lat_lon, exposure_type='pm25', resolution='7day', aggregation='max', utcOffset='utc'):
        return self.client.cmaq.get_cmaq_getScores (
            exposureType = exposure_type,
            startDate = datetime.datetime.strptime(start_date, "%Y-%m-%d").date (),
            endDate = datetime.datetime.strptime(end_date, "%Y-%m-%d").date (),
	    latLon = lat_lon,
            resolution = resolution,
            aggregation = aggregation,
            utcOffset = utcOffset).result (timeout = 10)

    def get_values (self, start_date, end_date, lat_lon, exposure_type='pm25', resolution='7day', aggregation='max', utcOffset='utc'):
        return self.client.cmaq.get_cmaq_getValues (
            exposureType = exposure_type,
            startDate = datetime.datetime.strptime(start_date, "%Y-%m-%d").date (),
            endDate = datetime.datetime.strptime(end_date, "%Y-%m-%d").date (),
	    latLon = lat_lon,
            resolution = resolution,
            aggregation = aggregation,
            utcOffset = utcOffset).result ()

#c = CMAQ ("https://exposures.renci.org/v1/swagger.json")
#c = CMAQ ("https://raw.githubusercontent.com/RENCI/nih-exposures-api/master/specification/swagger.yml")


#c = CMAQ ("https://app.swaggerhub.com/apiproxy/schema/file/mjstealey/environmental_exposures_api/0.0.1/swagger.json")
#c.inspect ()

#pprint (c.get_scores (start_date = "2011-01-01",
#                      end_date = "2011-12-31",
#                      lat_lon = "35.9131996,-79.0558445"))

#pprint (c.get_values (start_date = "2011-01-01",
#                      end_date = "2011-12-31",
#                      lat_lon = "35.9131996,-79.0558445"))

'''
{
  "cmaq": [
    {
      "aggregation": [
        [
          "max",
          "avg"
        ]
       ],
      "endDate": "2011-12-31",
      "exposureType": "pm25",
      "exposureUnit": "ugm3",
      "resolution": [
        [
          "hour",
          "day",
          "7day",
          "14day"
        ]
      ],
      "startDate": "2011-01-01"
    },
    {
      "aggregation": [
        [
          "max",
          "avg"
        ]
      ],
      "endDate": "2011-12-31",
      "exposureType": "o3",
      "exposureUnit": "ppm",
      "resolution": [
        [
          "hour",
          "day",
          "7day",
          "14day"
        ]
      ],
      "startDate": "2011-01-01"
    }
  ]
}


{
  "values": [
    {
      "dateTime": "2011-01-01T00:00:00+00:00",
      "latLon": "\"35.9131996,-79.0558445\"",
      "value": 30.3144
    },
    {
      "dateTime": "2011-01-02T00:00:00+00:00",
      "latLon": "\"35.9131996,-79.0558445\"",
      "value": 14.7758
    },
'''



from pyld import jsonld
import json
import requests
import sys

json_ld_context = sys.argv[1]
query_string = sys.argv[2]

context = None
with open (json_ld_context, "r") as stream:
    context = json.loads (stream.read ())

r = requests.get (query_string).json ()

text = json.dumps (r)

docs = r['data']

#print ("context: {}".format (context))

expanded = jsonld.expand (docs, { 'expandContext' : context } )
print ("{}".format (json.dumps (expanded, indent=2)))

compacted = jsonld.compact (expanded, context)
print ("{}".format (json.dumps (compacted, indent=2)))



import json
from pprint import pformat
from pprint import pprint
from greent.client import GraphQL
from collections import namedtuple
translator = GraphQL ("http://localhost:5000/graphql")

Translation = namedtuple ('Translation', [ 'thing', 'domain_a', 'domain_b' ])
translations = [
    Translation ("Imatinib", "http://chem2bio2rdf.org/drugbank/resource/Generic_Name", "http://chem2bio2rdf.org/uniprot/resource/gene"),
    Translation ("CDC25A",   "http://chem2bio2rdf.org/uniprot/resource/gene",          "http://chem2bio2rdf.org/kegg/resource/kegg_pathway"),
    Translation ("CACNA1A",  "http://chem2bio2rdf.org/uniprot/resource/gene",          "http://pharos.nih.gov/identiier/disease/name"),    
]

for translation in translations:
    result = translator.translate (
        thing=translation.thing,
        domain_a=translation.domain_a,
        domain_b=translation.domain_b)
    pprint ("{0} => {1}".format (translation.thing, pformat (result)))
